#!/usr/bin/python

import os, re
from urllib import parse
from urllib.request import pathname2url
from suds.client import Client
from suds.transport.http import HttpAuthenticated


#import logging
#logging.basicConfig(level=logging.INFO)
#logging.getLogger('suds.client').setLevel(logging.DEBUG)
#logging.getLogger('suds.transport').setLevel(logging.DEBUG)
#logging.getLogger('suds.xsd.schema').setLevel(logging.DEBUG)
#logging.getLogger('suds.wsdl').setLevel(logging.DEBUG)


class NetworkAutomationSOAPConnection:
    def __init__(self, hostUrl, username, password, wsdlFile):
        
        host = hostUrl+'/soap'
        wsdl_file = parse.urljoin('file:', pathname2url(os.path.abspath(wsdlFile)))
        t = HttpAuthenticated(username=username, password=password)
    
        client = Client(wsdl_file, transport=t)
        client.set_options(location=host)  
    
        request = client.factory.create('loginInputParms')
        request.username = username
        request.password = password
        authObject = client.service.login(request)
        self.sessionID = authObject.Text
        self.client = client
        #for method in client.wsdl.services[0].ports[0].methods.values():
            #print( '%s(%s)' % (method.name, ', '.join('%s: %s' % (part.type, part.name) for part in method.soap.input.body.parts)))
            #print(method.name)
            #print(method.soap.input.body.parts[0].type[0])
        #print(dir(self.client.factory.create('list_deviceInputParms')))

    def execute(self, methodName, paramDic=None):
        request = self.client.factory.create(methodName+'InputParms')
        request.sessionid = self.sessionID
        if paramDic != None:
            for k, v in paramDic.items():
                request[k] = v
        return getattr(self.client.service, methodName)(request)
    
    pattern = re.compile('<wsdl:operation name="(.*?)">', re.MULTILINE)
    def listDefinitions(self):
        cmds = ''
        with open(self.wsdlFilePath) as f:
            s = f.read()
            for match in self.pattern.finditer(s):
                commandName = match.groups()
                cmds += commandName + '\n'
        return cmds


#import pdb
def checkForErrors(result):
    #pdb.set_trace()
    status = result.Status
    if status == u'222' or status == u'GEN_FAILURE' or 'Error' in status:
        raise Exception( result )