'''
Created on Jul 11, 2017

@author: saul.vega
'''


while True:
    answer = input("Do you want to use the file credentials.py (y/n): ")
    if answer == 'y' or answer == 'Y':
        from credentials import *
        naVersion = soappyFile[len("SOAPbinding/hpna"):-1*len("_api.wsdl.soappy")].replace('_', '.')
        break
    elif answer == 'n' or answer == 'N':
        myserver = 'http://' + input("Server IP: ")
        naVersion = input("NA version: ")
        myuser = input("User: ")
        import getpass
        mypassword = getpass.getpass()
        soappyFile = 'SOAPbinding/hpna{0}_api.wsdl.soappy'.format(naVersion.replace('.', '_'))
        break
    print("Bad Option!\n")


usage = """
How to use this CLI:

1. Call the API call:
    <NA API command>

    For example:
    list device

2. Filter the output, 
    brief <parameters separated by comma without space> <api command>

    For example:
    brief hostName,driverName list device

3. Help
    help
    help related <NA API command>
    help parameters <NA API command>
 
    For example:
    help related list device
    

Examples:

help related list device
help parameters list device
list device -vendor Cisco

"""

import cmd, sys, re
from SOAPbinding import NetworkAutomationSOAPConnection
class NaShell(cmd.Cmd):
    """Simple command processor example."""
    
    intro = '\nNA-SOAP shell.   Type help to list commands.\n\tBy Saul Vega Chacon.\n'
    prompt = 'NA {0}> '.format(naVersion)
    file = None
    intRe = re.compile("^\d+$")
    
    def __init__(self):
        self.na = NetworkAutomationSOAPConnection(myserver, myuser, mypassword, soappyFile)
        sv_result = self.na.execute('show_version')
        if sv_result.Status == '200':
            print("Welcome to {0}!".format(myserver))
            print("\t" + sv_result.Text.replace("\n", "")) 
        
        
        self.completionDic = {}
        for method in self.na.client.wsdl.services[0].ports[0].methods.values():
            #print( '%s(%s)' % (method.name, ', '.join('%s: %s' % (part.type, part.name) for part in method.soap.input.body.parts)))
            #print(method.name)
            methodName = method.name
            params = method.soap.input.body.parts[0].type[0]
            self.completionDic[methodName.replace('_', ' ')] = dict(self.na.client.factory.create(params)).keys()
        cmd.Cmd.__init__(self)
    
    def do_exit(self, line):
        self.na.execute("logout")
        sys.exit(0)
        
    def do_py(self, line):
        try:
            eval(line)
        except Exception as e:
            print(e.__class__, ":", e)
            
    
        
    def emptyline(self):
        pass
    
    def showAvailableParameters(self, command):
        print('\n' + '-'*25)
        print("Command: " + command)
        print("\tParameters:", ', '.join(self.completionDic[command]))

    def do_help(self, line):
        if line == '':
            print(usage)
            return
        
        #try:
        action, command = line.split(" ", 1)
        #except ValueError:
        #    print(usage)
        #    return
        
        action = action.strip()
        command = command.strip()
            
        if action == 'related' and command == '':
            for x in self.completionDic.keys():
                print(x)
            print("exit --> to exit from CLI")
        elif action == 'related':
            print("Related commands:")
            for x in self.completionDic.keys():
                if x.startswith(command):
                    print(x)
        elif action == 'parameters':
            if command in self.completionDic:
                self.showAvailableParameters(command)
            else:
                print("{0}: not found!".format(command))
        else:
            print(usage)
    
    def runAPI(self, line):
        l = line.split("-")
        rawCommand = l.pop(0).strip()
        if rawCommand not in self.completionDic:
            print("'{0}': command not found!".format(rawCommand))
            return
        
        command = rawCommand.replace(' ', '_')
        availParamLst = self.completionDic[rawCommand]
        
        d = {}
        for x in l:
            param, value = x.split(' ', 1)
            value = value.strip()
            
            if param not in availParamLst:
                print("{0}: paramater not found!".format(param))
                self.showAvailableParameters(rawCommand)
                return
            if self.intRe.match(value):
                value = int(value)
            elif value[0] == '"' and value[-1] == '"':
                value = value[1:-1]
                
            d[param] = value
        
        #Here the code is executed
        return self.na.execute(command, d)
    
    def do_brief(self, line):
        """Usage of example:
        brief deviceid,hostname list device
        """
        varLst, apiCall = line.split(' ', 1)
        varLst = varLst.split(',')
        
        try:
            objLst = self.runAPI(apiCall).ResultSet[0]
        except Exception as e:
            print(e.__class__, ":", e)
            return
        
        l = []
        for obj in objLst:
            d = {}
            for var in varLst:
                if var in obj:
                    d[var] = obj[var]
            print(d)
        
    def default(self, line):
        print(self.runAPI(line))
    


if __name__ == '__main__':
    NaShell().cmdloop()






